#!/usr/bin/python

import socket, struct, sys
import message_pb2

class TrisDbConnection:
    def __init__(self, host, port = None):
        self.hostname = host
	self.port = port
	try:
	    if port is None:
	        # unix socket
		self.client = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
		self.client.connect(self.hostname)
	    else:
		# tcp
	    	self.client = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    	    	self.client.connect((self.hostname, self.port))
	except:
	    print "Error connecting to %s" % (self.hostname)
	    sys.exit()

    def get(self, s=None, p=None, o=None):
        req = message_pb2.QueryRequest()
	if s is None:
	    s = "*"
	if p is None:
            p = "*"
	if o is None:
            o = "*"
        req.query.append('GET "%s" "%s" "%s"' % (s, p, o))
	tmp = self.__sendData(req)
        result = []
        for i in tmp.data:
            result.append({'subject':i.subject, 'predicate':i.predicate, 'object':i.object})
        return result

    def gets(self, s, p, o):
        req = message_pb2.QueryRequest()
        req.query.append('GETS "%s" "%s" "%s"' % (s, p, o))
        tmp = self.__sendData(req)
        result = []
        for i in tmp.data:
            result.append(i.subject)
        return result

    def getp(self, s, p, o):
        req = message_pb2.QueryRequest()
        req.query.append('GETP "%s" "%s" "%s"' % (s, p, o))
        tmp = self.__sendData(req)
        result = []
        for i in tmp.data:
            result.append(i.predicate)
        return result

    def geto(self, s, p, o):
        req = message_pb2.QueryRequest()
        req.query.append('GETO "%s" "%s" "%s"' % (s, p, o))
        tmp = self.__sendData(req)
        result = []
        for i in tmp.data:
            result.append(i.object)
        return result

    def create(self, s, p, o):
        req = message_pb2.QueryRequest()
        q = req.query.append('CREATE "%s" "%s" "%s"' % (s, p, o))
	self.__sendData(req)
        return 'OK'

    def multi(self):
	return Multi()

    def execute(self, multi):
	self.__sendData(multi.request)
        return 'OK'

    def delete(self, s, p, o):
        req = message_pb2.QueryRequest()
        q = req.query.append('DELETE "%s" "%s" "%s"' % (s, p, o))
        self.__sendData(req)
        return 'OK'

    def clear(self):
        req = message_pb2.QueryRequest()
        req.query.append('CLEAR')
	self.__sendData(req)
	return 'OK'

    def save(self):
	req = message_pb2.QueryRequest()
	req.query.append('SAVE')
	self.__sendData(req)
	return 'OK'

    def count(self):
	req = message_pb2.QueryRequest()
	req.query.append('COUNT')
	tmp = self.__sendData(req)
	return tmp.data[0].object

    def close(self):
	self.client.close()

    def __sendData(self, request):
	try:
	    request.timestamp = '123456789'
	    s = request.SerializeToString()
	    # prepend header and send message
	    packed_len = struct.pack('>L', len(s))
            packed_message = packed_len + s
            self.client.send(packed_message)
	    # read header
	    len_buf = self.socket_read_n(4)
            msg_len = struct.unpack('>L', len_buf)[0]
            msg_buf = self.socket_read_n(msg_len)
            res = message_pb2.QueryResponse()
            res.ParseFromString(msg_buf)
	    return res
        except Exception as e:
            print "Connection error: ",e

    def socket_read_n(self, n):
        buf = ''
        while n > 0:
            data = self.client.recv(n)
            if data == '':
                raise RuntimeError('Connection error')
		sys.exit()
            buf += data
            n -= len(data)
        return buf


class Multi:
    def __init__(self):
        self.request = message_pb2.QueryRequest()

    def create(self, s, p, o):
	self.request.query.append('CREATE "%s" "%s" "%s"' % (s, p, o))



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: message.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)




DESCRIPTOR = _descriptor.FileDescriptor(
  name='message.proto',
  package='',
  serialized_pb='\n\rmessage.proto\"0\n\x0cQueryRequest\x12\r\n\x05query\x18\x01 \x03(\t\x12\x11\n\ttimestamp\x18\x02 \x01(\t\"\x85\x01\n\rQueryResponse\x12#\n\x04\x64\x61ta\x18\x01 \x03(\x0b\x32\x15.QueryResponse.Record\x12\x11\n\ttimestamp\x18\x02 \x01(\t\x1a<\n\x06Record\x12\x0f\n\x07subject\x18\x01 \x02(\t\x12\x11\n\tpredicate\x18\x02 \x02(\t\x12\x0e\n\x06object\x18\x03 \x02(\t')




_QUERYREQUEST = _descriptor.Descriptor(
  name='QueryRequest',
  full_name='QueryRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='query', full_name='QueryRequest.query', index=0,
      number=1, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='timestamp', full_name='QueryRequest.timestamp', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=17,
  serialized_end=65,
)


_QUERYRESPONSE_RECORD = _descriptor.Descriptor(
  name='Record',
  full_name='QueryResponse.Record',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='subject', full_name='QueryResponse.Record.subject', index=0,
      number=1, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='predicate', full_name='QueryResponse.Record.predicate', index=1,
      number=2, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='object', full_name='QueryResponse.Record.object', index=2,
      number=3, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=141,
  serialized_end=201,
)

_QUERYRESPONSE = _descriptor.Descriptor(
  name='QueryResponse',
  full_name='QueryResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='data', full_name='QueryResponse.data', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='timestamp', full_name='QueryResponse.timestamp', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[_QUERYRESPONSE_RECORD, ],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=68,
  serialized_end=201,
)

_QUERYRESPONSE_RECORD.containing_type = _QUERYRESPONSE;
_QUERYRESPONSE.fields_by_name['data'].message_type = _QUERYRESPONSE_RECORD
DESCRIPTOR.message_types_by_name['QueryRequest'] = _QUERYREQUEST
DESCRIPTOR.message_types_by_name['QueryResponse'] = _QUERYRESPONSE

class QueryRequest(_message.Message):
  __metaclass__ = _reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _QUERYREQUEST

  # @@protoc_insertion_point(class_scope:QueryRequest)

class QueryResponse(_message.Message):
  __metaclass__ = _reflection.GeneratedProtocolMessageType

  class Record(_message.Message):
    __metaclass__ = _reflection.GeneratedProtocolMessageType
    DESCRIPTOR = _QUERYRESPONSE_RECORD

    # @@protoc_insertion_point(class_scope:QueryResponse.Record)
  DESCRIPTOR = _QUERYRESPONSE

  # @@protoc_insertion_point(class_scope:QueryResponse)


# @@protoc_insertion_point(module_scope)



from distutils.core import setup

setup(
    name = 'trisdb-py',
    py_modules = ['trisdb', 'message_pb2'],
    version = '1.0.0',
    description = 'The Python interface to TrisDb',
    author = 'Gianluca Tiepolo',
    author_email = 'tiepolo.gian@gmail.com',
    long_description=open('README.md').read(),
    url = 'https://github.com/tiepologian/trisdb-py',
    keywords = ['trisdb', 'python'],
    classifiers = [],
    install_requires=[
"protobuf",
    ],
)
